@@ -69,6 +69,7 @@
 // #![feature(const_trait_impl)]
 // #![feature(const_fn_trait_bound)]
 // #![feature(const_fn_fn_ptr_basics)]
+#![feature(crate_visibility_modifier)]
 #![forbid(unsafe_op_in_unsafe_fn)]
 #![warn(clippy::missing_const_for_fn)]
 #![no_std]
@@ -92,4 +93,4 @@ pub use key::Key;
 
 mod test_index;
 
-pub use test_index::{FromIndex, IntoIndex};
+pub use test_index::{FromIndex, Idx, IntoIndex, NewIndex};
