@@ -11,9 +11,7 @@ use core::{
     ops::{Index, IndexMut},
 };
 
-use NonZeroUsize as Nzu;
-
-use crate::{FromIndex, IntoIndex, Key};
+use crate::{FromIndex, Idx, IntoIndex, Key, NewIndex};
 
 mod hasher;
 
@@ -96,7 +94,7 @@ const fn ceil_log_2(x: NonZeroUsize) -> usize {
 //     ];
 //     for (input, output) in inputs.iter().copied() {
 //         assert_eq!(
-//             ceil_log_2(Nzu::new(input).unwrap()),
+//             ceil_log_2(NonZeroUsize::new(input).unwrap()),
 //             output,
 //             "input = {}. lz = {}",
 //             input,
@@ -199,9 +197,7 @@ type Nums = [usize; NUMS_LEN];
 /// [`with_capacity_and_hasher`]: Self::with_capacity_and_hasher
 ///
 /// ```
-/// use core::num::NonZeroUsize;
-///
-/// use hash_arr_map::{FromIndex, Ham, IntoIndex};
+/// use hash_arr_map::{FromIndex, Ham, Idx, IntoIndex};
 ///
 /// #[derive(Hash, Eq, PartialEq, Debug)]
 /// struct Item {
@@ -217,17 +213,17 @@ type Nums = [usize; NUMS_LEN];
 /// }
 ///
 /// impl IntoIndex for Item {
-///     fn into_index(&self) -> Option<NonZeroUsize> {
-///         NonZeroUsize::new(((self.cents as usize) << 8) + self.dollars as usize)
+///     fn into_index(&self) -> Option<Idx<Self>> {
+///         unsafe { Idx::from_usize(((self.cents as usize) << 8) + self.dollars as usize) }
 ///     }
 /// }
 ///
 /// impl FromIndex for Item {
-///     fn try_from_index(idx: NonZeroUsize) -> Option<Self> {
-///         Some(Self {
-///             dollars: idx.get() as u8,
-///             cents: (idx.get() >> 8) as u8,
-///         })
+///     fn from_index(idx: Idx<Self>) -> Self {
+///         Self {
+///             dollars: idx.get().get() as u8,
+///             cents: (idx.get().get() >> 8) as u8,
+///         }
 ///     }
 /// }
 ///
@@ -534,6 +530,58 @@ impl<K: FromIndex, V> Ham<K, V> {
 }
 
 impl<K: FromIndex + Hash + Eq, V> Ham<K, V> {
+    #[must_use]
+    pub fn get_key_value<Q: ?Sized>(&self, key: &Q) -> Option<(Key<'_, K>, &V)>
+    where
+        K: Borrow<Q>,
+        Q: IntoIndex + Hash + Eq,
+    {
+        if let Some((idx, v)) = self.get_array_idx(key) {
+            v.as_ref()
+                .map(|v| (Key::from_index(unsafe { idx.cast() }), v))
+        } else {
+            self.get_hash(key)
+                .and_then(|v| v.value().as_ref().map(|(k, v)| (Key::Borrowed(k), v)))
+        }
+    }
+
+    /// Removes a key from the map, returning the stored key and value if the
+    /// key was previously in the map.
+    ///
+    /// The key may be any borrowed form of the map's key type, but Hash and Eq
+    /// on the borrowed form must match those for the key type.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use hash_arr_map::Ham;
+    ///
+    /// let mut map = Ham::new();
+    /// map.insert(1, "a");
+    /// assert_eq!(map.remove_entry(&1), Some((1, "a")));
+    /// assert_eq!(map.remove(&1), None);
+    /// ```
+    #[must_use]
+    pub fn remove_entry<Q: ?Sized>(&mut self, key: &Q) -> Option<(K, V)>
+    where
+        K: Borrow<Q>,
+        Q: IntoIndex + Hash + Eq,
+    {
+        let v = if let Some((idx, v)) = self.get_array_idx_mut(key) {
+            v.take().map(|v| (K::from_index(unsafe { idx.cast() }), v))
+        } else {
+            self.get_hash_take(key)
+        };
+
+        if v.is_some() {
+            self.len -= 1;
+        }
+
+        v
+    }
+}
+
+impl<K: NewIndex + Hash + Eq, V> Ham<K, V> {
     /// This function finds a boundary in the `table`, such that `table.get(n)`
     /// is `Some(_)`, and `table.get(n + 1)` is `None`.
     pub fn get_n(&self) -> Option<usize> {
@@ -573,7 +621,7 @@ impl<K: FromIndex + Hash + Eq, V> Ham<K, V> {
             let mut max = array_len + 1;
 
             while self
-                .get_hash(&K::try_from_index(unsafe { Nzu::new_unchecked(max) })?)
+                .get_hash(&Idx::try_new_usize(max)?.into_value())
                 .is_some()
             {
                 if max == usize::MAX {
@@ -585,61 +633,12 @@ impl<K: FromIndex + Hash + Eq, V> Ham<K, V> {
 
             binary_search(min, max, |i| {
                 Some(
-                    self.get_hash(&K::try_from_index(unsafe { Nzu::new_unchecked(i) })?)
+                    self.get_hash(&Idx::try_new_usize(i)?.into_value())
                         .is_some(),
                 )
             })
         }
     }
-
-    #[must_use]
-    pub fn get_key_value<Q: ?Sized>(&self, key: &Q) -> Option<(Key<'_, K>, &V)>
-    where
-        K: Borrow<Q>,
-        Q: IntoIndex + Hash + Eq,
-    {
-        if let Some((idx, v)) = self.get_array_idx(key) {
-            v.as_ref().map(|v| (unsafe { Key::from_index(idx) }, v))
-        } else {
-            self.get_hash(key)
-                .and_then(|v| v.value().as_ref().map(|(k, v)| (Key::Borrowed(k), v)))
-        }
-    }
-
-    /// Removes a key from the map, returning the stored key and value if the
-    /// key was previously in the map.
-    ///
-    /// The key may be any borrowed form of the map's key type, but Hash and Eq
-    /// on the borrowed form must match those for the key type.
-    ///
-    /// # Examples
-    ///
-    /// ```rust
-    /// use hash_arr_map::Ham;
-    ///
-    /// let mut map = Ham::new();
-    /// map.insert(1, "a");
-    /// assert_eq!(map.remove_entry(&1), Some((1, "a")));
-    /// assert_eq!(map.remove(&1), None);
-    /// ```
-    #[must_use]
-    pub fn remove_entry<Q: ?Sized>(&mut self, key: &Q) -> Option<(K, V)>
-    where
-        K: Borrow<Q>,
-        Q: IntoIndex + Hash + Eq,
-    {
-        let v = if let Some((idx, v)) = self.get_array_idx_mut(key) {
-            v.take().map(|v| (unsafe { K::from_index(idx) }, v))
-        } else {
-            self.get_hash_take(key)
-        };
-
-        if v.is_some() {
-            self.len -= 1;
-        }
-
-        v
-    }
 }
 
 impl<K: IntoIndex + FromIndex + Hash + Eq, V> Ham<K, V> {
@@ -872,7 +871,7 @@ impl<K: IntoIndex + FromIndex + Hash + Eq, V> Ham<K, V> {
     // @@  @@  @@@@@@  @@  @@  @@  @@  @@@@@@  @@  @@
     fn count_int(key: &K, nums: &mut Nums) -> bool {
         if let Some(v) = key.into_index() {
-            nums[ceil_log_2(v)] += 1;
+            nums[ceil_log_2(v.get())] += 1;
             true
         } else {
             false
@@ -1042,7 +1041,7 @@ impl<K: IntoIndex + FromIndex + Hash + Eq, V> Ham<K, V> {
             for (i, item) in array.drain(array_part..).enumerate() {
                 if let Some(item) = item {
                     self.new_key(
-                        unsafe { K::from_index(Nzu::new_unchecked(array_part + i + 1)) },
+                        K::from_index(unsafe { Idx::new_unchecked(array_part + i + 1) }),
                         item,
                     ); // Insert into the hash part.
                 }
@@ -1079,31 +1078,32 @@ impl<K, V> Ham<K, V> {
     // @@  @@  @@        @@      @@    @@      @@  @@      @@
     // @@@@@@  @@@@@@    @@      @@    @@@@@@  @@  @@  @@@@@@
     fn get_array_idx_only<Q: ?Sized + IntoIndex>(&self, key: &Q) -> Option<usize> {
-        let v = key.into_index()?.get() - 1;
+        let v = key.into_index()?.get().get() - 1;
 
         self.array.get(v).map(|_| v)
     }
 
-    fn get_array_idx<Q: ?Sized + IntoIndex>(&self, key: &Q) -> Option<(Nzu, &Option<V>)> {
+    fn get_array_idx<Q: ?Sized + IntoIndex>(&self, key: &Q) -> Option<(Idx<Q>, &Option<V>)> {
         key.into_index()
-            .and_then(|v| Some((v, self.array.get(v.get() - 1)?)))
+            .and_then(|v| Some((v.clone(), self.array.get(v.get().get() - 1)?)))
     }
 
     fn get_array_idx_mut<Q: ?Sized + IntoIndex>(
         &mut self,
         key: &Q,
-    ) -> Option<(Nzu, &mut Option<V>)> {
+    ) -> Option<(Idx<Q>, &mut Option<V>)> {
         key.into_index()
-            .and_then(move |v| Some((v, self.array.get_mut(v.get() - 1)?)))
+            .and_then(move |v| Some((v.clone(), self.array.get_mut(v.get().get() - 1)?)))
     }
 
     fn get_array<Q: ?Sized + IntoIndex>(&self, key: &Q) -> Option<&Option<V>> {
-        key.into_index().and_then(|v| self.array.get(v.get() - 1))
+        key.into_index()
+            .and_then(|v| self.array.get(v.get().get() - 1))
     }
 
     fn get_array_mut<Q: ?Sized + IntoIndex>(&mut self, key: &Q) -> Option<&mut Option<V>> {
         key.into_index()
-            .and_then(move |v| self.array.get_mut(v.get() - 1))
+            .and_then(move |v| self.array.get_mut(v.get().get() - 1))
     }
 
     fn get_array_take<Q: ?Sized + IntoIndex>(&mut self, key: &Q) -> Option<V> {
@@ -1131,6 +1131,35 @@ impl<K, V> Ham<K, V> {
         }
     }
 
+    fn get_hash_mut_for_take<Q: ?Sized>(&mut self, key: &Q) -> Option<(&mut Node<K, V>, bool)>
+    where
+        Q: Hash + Eq,
+        K: Borrow<Q>,
+    {
+        if self.hash.is_empty() {
+            None
+        } else {
+            let mut v = Self::main_position_cell(self as *mut _, key);
+
+            let mut prev_v = None;
+            while !Node::matches(v, key) {
+                let new_v = Node::next(v);
+                if new_v as *const _ == v {
+                    return None; // There wasn't a chain.
+                } else {
+                    prev_v = Some(v);
+                    v = new_v;
+                }
+            }
+
+            Some(if let Some(prev) = prev_v {
+                (unsafe { &mut *prev.as_ptr() }, true)
+            } else {
+                (unsafe { &mut *v.as_ptr() }, false)
+            })
+        }
+    }
+
     fn get_hash_take<Q: ?Sized>(&mut self, key: &Q) -> Option<(K, V)>
     where
         Q: Hash + Eq,
@@ -1167,7 +1196,7 @@ impl<K, V> Ham<K, V> {
                 if v_next as *const _ != v {
                     // prev_v -> v -> v_next -> ...
                     Node::set_next(prev_v, v_next);
-                // prev_v -> v_next -> ...; v -> v_next -> ...
+                    // prev_v -> v_next -> ...; v -> v_next -> ...
                 } else {
                     // prev_v -> v
                     Node::set_next(prev_v, prev_v);
@@ -1349,7 +1378,7 @@ where
     where
         I: IntoIterator<Item = (K, V)>,
     {
-        let mut map = Ham::new();
+        let mut map = Self::new();
         map.extend(iter);
         map
     }
